@@ -77,7 +77,7 @@ know:
 #include "config.h"
 #include "src/tlsdate-helper.h"
 
-#include "proxy-bio.h"
+#include "src/proxy-bio.h"
 
 #include "src/compat/clock.h"
 
@@ -210,7 +210,7 @@ void
 openssl_time_callback (const SSL* ssl, int where, int ret)
 {
   if (where == SSL_CB_CONNECT_LOOP &&
-      (ssl->state == SSL3_ST_CR_CERT_A || ssl->state == SSL3_ST_CR_CERT_B))
+      (ssl->state == SSL3_ST_CR_SRVR_HELLO_A || ssl->state == SSL3_ST_CR_SRVR_HELLO_B))
   {
     // XXX TODO: If we want to trust the remote system for time,
     // can we just read that time out of the remote system and if the
@@ -231,7 +231,8 @@ openssl_time_callback (const SSL* ssl, int where, int ret)
       verb("V: remote peer provided: %d, preferred over compile time: %d\n",
             ntohl(server_time), compiled_time);
       verb("V: freezing time with X509_VERIFY_PARAM_set_time\n");
-      X509_VERIFY_PARAM_set_time(ssl->ctx->param, (time_t) ntohl(server_time));
+      X509_VERIFY_PARAM_set_time(ssl->ctx->cert_store->param,
+                                 (time_t) ntohl(server_time) + 86400);
     } else {
       die("V: the remote server is a false ticker! server: %d compile: %d\n",
            ntohl(server_time), compiled_time);
@@ -517,6 +518,9 @@ check_san (SSL *ssl, const char *hostname)
 
         STACK_OF(CONF_VALUE) *val;
         CONF_VALUE *nval;
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
+        const
+#endif
         X509V3_EXT_METHOD *method;
 
         if (!(method = X509V3_EXT_get(ext)))
@@ -679,6 +683,7 @@ run_ssl (uint32_t *time_map, int time_is_an_illusion)
   BIO *s_bio;
   SSL_CTX *ctx;
   SSL *ssl;
+  struct stat statbuf;
 
   SSL_load_error_strings();
   SSL_library_init();
@@ -704,8 +709,25 @@ run_ssl (uint32_t *time_map, int time_is_an_illusion)
 
   if (ca_racket)
   {
-    if (1 != SSL_CTX_load_verify_locations(ctx, NULL, certdir))
-      fprintf(stderr, "SSL_CTX_load_verify_locations failed\n");
+    if (-1 == stat(ca_cert_container, &statbuf))
+    {
+      die("Unable to stat CA certficate container\n");
+    } else
+    {
+      switch (statbuf.st_mode & S_IFMT)
+      {
+      case S_IFREG:
+        if (1 != SSL_CTX_load_verify_locations(ctx, ca_cert_container, NULL))
+          fprintf(stderr, "SSL_CTX_load_verify_locations failed\n");
+        break;
+      case S_IFDIR:
+        if (1 != SSL_CTX_load_verify_locations(ctx, NULL, ca_cert_container))
+          fprintf(stderr, "SSL_CTX_load_verify_locations failed\n");
+        break;
+      default:
+        die("Unable to load CA certficate container\n");
+      }
+    }
   }
 
   if (NULL == (s_bio = make_ssl_bio(ctx)))
@@ -750,53 +772,6 @@ run_ssl (uint32_t *time_map, int time_is_an_illusion)
 }
 
 /** drop root rights and become 'nobody' */
-static void
-become_nobody (void)
-{
-  uid_t uid;
-  gid_t gid;
-  struct passwd *pw;
-  struct group  *gr;
-
-  if (0 != getuid ())
-    return; /* not running as root to begin with; should (!) be harmless to continue
-         without dropping to 'nobody' (setting time will fail in the end) */
-  pw = getpwnam(UNPRIV_USER);
-  gr = getgrnam(UNPRIV_GROUP);
-  if (NULL == pw)
-    die ("Failed to obtain UID for `%s'\n", UNPRIV_USER);
-  if (NULL == gr)
-    die ("Failed to obtain GID for `%s'\n", UNPRIV_GROUP);
-  uid = pw->pw_uid;
-  if (0 == uid)
-    die ("UID for `%s' is 0, refusing to run SSL\n", UNPRIV_USER);
-  gid = pw->pw_gid;
-  if (0 == gid || 0 == gr->gr_gid)
-    die ("GID for `%s' is 0, refusing to run SSL\n", UNPRIV_USER);
-  if (pw->pw_gid != gr->gr_gid)
-    die ("GID for `%s' is not `%s' as expected, refusing to run SSL\n",
-          UNPRIV_USER, UNPRIV_GROUP);
-
-  if (0 != initgroups((const char *)UNPRIV_USER, gr->gr_gid))
-    die ("Unable to initgroups for `%s' in group `%s' as expected\n",
-          UNPRIV_USER, UNPRIV_GROUP);
-
-#ifdef HAVE_SETRESGID
-  if (0 != setresgid (gid, gid, gid))
-    die ("Failed to setresgid: %s\n", strerror (errno));
-#else
-  if (0 != (setgid (gid) | setegid (gid)))
-    die ("Failed to setgid: %s\n", strerror (errno));
-#endif
-#ifdef HAVE_SETRESUID
-  if (0 != setresuid (uid, uid, uid))
-    die ("Failed to setresuid: %s\n", strerror (errno));
-#else
-  if (0 != (setuid (uid) | seteuid (uid)))
-    die ("Failed to setuid: %s\n", strerror (errno));
-#endif
-}
-
 
 int
 main(int argc, char **argv)
@@ -818,7 +793,7 @@ main(int argc, char **argv)
   hostname_to_verify = argv[1];
   port = argv[2];
   protocol = argv[3];
-  certdir = argv[6];
+  ca_cert_container = argv[6];
   ca_racket = (0 != strcmp ("unchecked", argv[4]));
   verbose = (0 != strcmp ("quiet", argv[5]));
   setclock = (0 == strcmp ("setclock", argv[7]));
@@ -839,7 +814,7 @@ main(int argc, char **argv)
   /* We are not going to set the clock, thus no need to stay root */
   if (0 == setclock && 0 == timewarp)
   {
-    become_nobody ();
+    drop_privs_to (UNPRIV_USER, UNPRIV_GROUP);
   }
 
   time_map = mmap (NULL, sizeof (uint32_t),
@@ -897,11 +872,11 @@ main(int argc, char **argv)
     die ("fork failed: %s\n", strerror (errno));
   if (0 == ssl_child)
   {
-    become_nobody ();
+    drop_privs_to (UNPRIV_USER, UNPRIV_GROUP);
     run_ssl (time_map, leap);
     (void) munmap (time_map, sizeof (uint32_t));
     _exit (0);
-  } 
+  }
   if (ssl_child != waitpid (ssl_child, &status, 0))
     die ("waitpid failed: %s\n", strerror (errno));
   if (! (WIFEXITED (status) && (0 == WEXITSTATUS (status)) ))
@@ -909,7 +884,7 @@ main(int argc, char **argv)
 
   if (0 != clock_get_real_time(&end_time))
     die ("Failed to read current time of day: %s\n", strerror (errno));
-  
+
   /* calculate RTT */
   rt_time_ms = (CLOCK_SEC(&end_time) - CLOCK_SEC(&start_time)) * 1000 + (CLOCK_USEC(&end_time) - CLOCK_USEC(&start_time)) / 1000;
   if (rt_time_ms < 0)
